@@ -20,7 +20,7 @@ import re
 
 import gtk
 from gtk import gdk
-import gnomevfs
+import gio
 import gedit
 
 from Library import Library
@@ -42,6 +42,7 @@ class Document:
                 self.active_placeholder = None
                 self.signal_ids = {}
                 
+                self.ordered_placeholders = []
                 self.update_placeholders = []
                 self.jump_placeholders = []
                 self.language_id = 0
@@ -74,6 +75,10 @@ class Document:
         def connect_signal(self, obj, signal, cb):
                 self.disconnect_signal(obj, signal)     
                 self.signal_ids[(obj, signal)] = obj.connect(signal, cb)
+
+        def connect_signal_after(self, obj, signal, cb):
+                self.disconnect_signal(obj, signal)
+                self.signal_ids[(obj, signal)] = obj.connect_after(signal, cb)
                 
         # Set the view to be controlled. Installs signal handlers and sets current
         # language. If there is already a view set this function will first remove
@@ -87,7 +92,7 @@ class Document:
                         # Remove signals
                         signals = {self.view: ('key-press-event', 'destroy', 
                                                'notify::editable', 'drag-data-received'),
-                                   buf:       ('notify::language', 'changed', 'cursor-moved')}
+                                   buf:       ('notify::language', 'changed', 'cursor-moved', 'insert-text')}
                         
                         for obj, sig in signals.items():
                                 for s in sig:
@@ -113,9 +118,15 @@ class Document:
 
                         self.update_language()
                 elif self.language_id != 0:
-                        Library().unref(self.language_id)
-                        self.language_id = 0
-        
+                        langid = self.language_id
+                        
+                        self.language_id = None;
+                        
+                        if self.instance:
+                                self.instance.language_changed(self)
+
+                        Library().unref(langid)
+
         def set_view(self, view):
                 if view == self.view:
                         return
@@ -132,8 +143,7 @@ class Document:
                 elif lang and lang.get_id() == self.language_id:
                         return
 
-                if self.language_id != 0:
-                        Library().unref(self.language_id)
+                langid = self.language_id
 
                 if lang:
                         self.language_id = lang.get_id()
@@ -143,6 +153,9 @@ class Document:
                 if self.instance:
                         self.instance.language_changed(self)
 
+                if langid != 0:
+                        Library().unref(langid)
+
                 Library().ref(self.language_id)
 
         def accelerator_activate(self, keyval, mod):
@@ -170,67 +183,95 @@ class Document:
                 
                 self.connect_signal(buf, 'changed', self.on_buffer_changed)
                 self.connect_signal(buf, 'cursor-moved', self.on_buffer_cursor_moved)
+                self.connect_signal_after(buf, 'insert-text', self.on_buffer_insert_text)
         
         def last_snippet_removed(self):
                 buf = self.view.get_buffer()
                 self.disconnect_signal(buf, 'changed')
                 self.disconnect_signal(buf, 'cursor-moved')
+                self.disconnect_signal(buf, 'insert-text')
 
         def current_placeholder(self):
                 buf = self.view.get_buffer()
                 
                 piter = buf.get_iter_at_mark(buf.get_insert())        
-                current = None
+                found = []
 
                 for placeholder in self.placeholders:
                         begin = placeholder.begin_iter()
                         end = placeholder.end_iter()
 
-                        if piter.compare(begin) >= 0 and \
-                                        piter.compare(end) <= 0:
-                                current = placeholder
+                        if piter.compare(begin) >= 0 and piter.compare(end) <= 0:
+                                found.append(placeholder)
 
-                return current
+                if self.active_placeholder in found:
+                        return self.active_placeholder
+                elif len(found) > 0:
+                        return found[0]
+                else:
+                        return None
 
         def advance_placeholder(self, direction):
                 # Returns (CurrentPlaceholder, NextPlaceholder), depending on direction
                 buf = self.view.get_buffer()
                 
                 piter = buf.get_iter_at_mark(buf.get_insert())
-                prev = current = next = None
-                length = len(self.placeholders)                
+                found = current = next = None
+                length = len(self.placeholders)
+                
+                placeholders = list(self.placeholders)
+                
+                if self.active_placeholder:
+                        begin = self.active_placeholder.begin_iter()
+                        end = self.active_placeholder.end_iter()
+                        
+                        if piter.compare(begin) >= 0 and piter.compare(end) <= 0:
+                                current = self.active_placeholder
+                                currentIndex = placeholders.index(self.active_placeholder)
 
                 if direction == 1:
-                        nearest = lambda w, x, y, z: (w.compare(y) >= 0 and (not z or \
-                                        y.compare(z.end_iter()) >= 0))
+                        # w = piter, x = begin, y = end, z = found
+                        nearest = lambda w, x, y, z: (w.compare(x) <= 0 and (not z or \
+                                        x.compare(z.begin_iter()) < 0))
                         indexer = lambda x: x < length - 1
                 else:
-                        nearest = lambda w, x, y, z: (w.compare(x) <= 0 and (not z or \
-                                        x.compare(z.end_iter()) <= 0))
+                        # w = piter, x = begin, y = end, z = prev
+                        nearest = lambda w, x, y, z: (w.compare(x) >= 0 and (not z or \
+                                        x.compare(z.begin_iter()) >= 0))
                         indexer = lambda x: x > 0
 
                 for index in range(0, length):
-                        placeholder = self.placeholders[index]
+                        placeholder = placeholders[index]
                         begin = placeholder.begin_iter()
                         end = placeholder.end_iter()
                         
                         # Find the nearest placeholder
-                        if nearest(piter, begin, end, prev):
-                                prevIndex = index
-                                prev = placeholder
+                        if nearest(piter, begin, end, found):
+                                foundIndex = index
+                                found = placeholder
                         
                         # Find the current placeholder
                         if piter.compare(begin) >= 0 and \
-                                        piter.compare(end) <= 0:
+                                        piter.compare(end) <= 0 and \
+                                        current == None:
                                 currentIndex = index
                                 current = placeholder
                 
+                if current and current != found and \
+                   (current.begin_iter().compare(found.begin_iter()) == 0 or \
+                    current.end_iter().compare(found.begin_iter()) == 0) and \
+                   self.active_placeholder and \
+                   current.begin_iter().compare(self.active_placeholder.begin_iter()) == 0:
+                        # if current and found are at the same place, then
+                        # resolve the 'hugging' problem
+                        current = self.active_placeholder
+                        currentIndex = placeholders.index(current)
+                
                 if current:
                         if indexer(currentIndex):
-                                next = self.placeholders[currentIndex + direction]
-                elif prev:
-                        if indexer(prevIndex):
-                                next = self.placeholders[prevIndex + direction]
+                                next = placeholders[currentIndex + direction]
+                elif found:
+                        next = found
                 elif length > 0:
                         next = self.placeholders[0]
                 
@@ -246,6 +287,9 @@ class Document:
                 buf = self.view.get_buffer()
                 self.view.scroll_mark_onscreen(buf.get_insert())
         
+        def set_active_placeholder(self, placeholder):
+                self.active_placeholder = placeholder
+
         def goto_placeholder(self, current, next):
                 last = None
 
@@ -256,7 +300,7 @@ class Document:
                         if current.__class__ == PlaceholderEnd:
                                 last = current
                 
-                self.active_placeholder = next
+                self.set_active_placeholder(next)
                 
                 if next:
                         next.enter()
@@ -267,7 +311,7 @@ class Document:
                 if last:
                         # This is the end of the placeholder, remove the snippet etc
                         for snippet in list(self.active_snippets):
-                                if snippet[3][0] == last:
+                                if snippet.placeholders[0] == last:
                                         self.deactivate_snippet(snippet)
                                         break
                 
@@ -295,30 +339,99 @@ class Document:
                 start, end = buffer_word_boundary(buf)
                 
                 return buf.get_text(start, end)
+
+        def env_get_current_line(self, buf):
+                start, end = buffer_line_boundary(buf)
                 
-        def env_get_filename(self, buf):
+                return buf.get_text(start, end)
+
+        def env_get_current_line_number(self, buf):
+                start, end = buffer_line_boundary(buf)
+                
+                return str(start.get_line() + 1)
+                
+        def env_get_document_uri(self, buf):
                 uri = buf.get_uri()
                 
                 if uri:
-                        return buf.get_uri_for_display()
+                        return uri
                 else:
                         return ''
         
-        def env_get_basename(self, buf):
+        def env_get_document_name(self, buf):
                 uri = buf.get_uri()
                 
                 if uri:
-                        return os.path.basename(buf.get_uri_for_display())
+                        return os.path.basename(uri)
                 else:
                         return ''
 
+        def env_get_document_scheme(self, buf):
+                uri = buf.get_uri()
+                
+                if uri:
+                        return gio.File(uri).get_uri_scheme()
+                else:
+                        return ''
+
+        def env_get_document_path(self, buf):
+                uri = buf.get_uri()
+                
+                if uri and gedit.utils.uri_has_file_scheme(uri):
+                        return gio.File(uri).get_path()
+                else:
+                        return ''
+
+        def env_get_document_dir(self, buf):
+                uri = buf.get_uri()
+                
+                if uri and gedit.utils.uri_has_file_scheme(uri):
+                        return os.path.dirname(gio.File(uri).get_path())
+                else:
+                        return ''
+
+        def env_get_document_type(self, buf):
+                typ = buf.get_mime_type()
+                
+                if typ:
+                        return typ
+                else:
+                        return ''
+
+        def env_get_documents_uri(self, buf):
+                documents_uri = [doc.get_uri()
+                                 for doc in self.view.get_toplevel().get_documents()
+                                 if doc.get_uri() is not None]
+                
+                return ' '.join(documents_uri)
+
+        def env_get_documents_path(self, buf):
+                documents_uri = [doc.get_uri()
+                                 for doc in self.view.get_toplevel().get_documents()
+                                 if doc.get_uri() is not None]
+
+                documents_path = [gio.File(uri).get_path()
+                                 for uri in documents_uri
+                                 if gedit.utils.uri_has_file_scheme(uri)]
+                
+                return ' '.join(documents_path)
+
         def update_environment(self):
                 buf = self.view.get_buffer()
                 
                 variables = {'GEDIT_SELECTED_TEXT': self.env_get_selected_text, 
                              'GEDIT_CURRENT_WORD': self.env_get_current_word, 
-                             'GEDIT_FILENAME': self.env_get_filename, 
-                             'GEDIT_BASENAME': self.env_get_basename}
+                             'GEDIT_CURRENT_LINE': self.env_get_current_line,
+                             'GEDIT_CURRENT_LINE_NUMBER': self.env_get_current_line_number,
+                             'GEDIT_CURRENT_DOCUMENT_URI': self.env_get_document_uri, 
+                             'GEDIT_CURRENT_DOCUMENT_NAME': self.env_get_document_name,
+                             'GEDIT_CURRENT_DOCUMENT_SCHEME': self.env_get_document_scheme,
+                             'GEDIT_CURRENT_DOCUMENT_PATH': self.env_get_document_path,
+                             'GEDIT_CURRENT_DOCUMENT_DIR': self.env_get_document_dir,
+                             'GEDIT_CURRENT_DOCUMENT_TYPE': self.env_get_document_type,
+                             'GEDIT_DOCUMENTS_URI': self.env_get_documents_uri,
+                             'GEDIT_DOCUMENTS_PATH': self.env_get_documents_path,
+                             }
                 
                 for var in variables:
                         os.environ[var] = variables[var](buf)
@@ -331,6 +444,15 @@ class Document:
                                 return True
                 
                 return False
+        
+        def uses_current_line(self, snippet):
+                matches = re.findall('(\\\\*)\\$GEDIT_CURRENT_LINE', snippet['text'])
+                
+                for match in matches:
+                        if len(match) % 2 == 0:
+                                return True
+                
+                return False
 
         def apply_snippet(self, snippet, start = None, end = None):
                 if not snippet.valid:
@@ -350,6 +472,11 @@ class Document:
                         # the current word. Set start and end to the word boundary so that 
                         # it will be removed
                         start, end = buffer_word_boundary(buf)
+                elif start.equal(end) and self.uses_current_line(s):
+                        # There is no tab trigger and no selection and the snippet uses
+                        # the current line. Set start and end to the line boundary so that 
+                        # it will be removed
+                        start, end = buffer_line_boundary(buf)
 
                 # Set environmental variables
                 self.update_environment()
@@ -366,26 +493,23 @@ class Document:
                 
                 # Insert the snippet
                 holders = len(self.placeholders)
-                active_info = s.insert_into(self, start)
-                self.active_snippets.append(active_info)
-
-                # Put cursor back to beginning of the snippet
-                piter = buf.get_iter_at_mark(active_info[0])
-                buf.place_cursor(piter)
-
-                # Jump to first placeholder
-                (current, next) = self.next_placeholder()
                 
-                if current and current != self.active_placeholder:
-                        self.goto_placeholder(None, current)
-                elif next:
-                        self.goto_placeholder(None, next)                
-                        
-                buf.end_user_action()
-                
-                if len(self.active_snippets) == 1:
+                if len(self.active_snippets) == 0:
                         self.first_snippet_inserted()
 
+                sn = s.insert_into(self, start)
+                self.active_snippets.append(sn)
+
+                # Put cursor at first tab placeholder
+                keys = filter(lambda x: x > 0, sn.placeholders.keys())
+                
+                if len(keys) == 0:
+                        buf.place_cursor(sn.begin_iter())
+                else:
+                        self.goto_placeholder(self.active_placeholder, sn.placeholders[keys[0]])
+                
+                buf.end_user_action()
+
                 return True
 
         def get_tab_tag(self, buf):
@@ -444,11 +568,11 @@ class Document:
                 buf = self.view.get_buffer()
                 remove = []
                 
-                for tabstop in snippet[3]:
+                for tabstop in snippet.placeholders:
                         if tabstop == -1:
-                                placeholders = snippet[3][-1]
+                                placeholders = snippet.placeholders[-1]
                         else:
-                                placeholders = [snippet[3][tabstop]]
+                                placeholders = [snippet.placeholders[tabstop]]
                         
                         for placeholder in placeholders:
                                 if placeholder in self.placeholders:
@@ -466,12 +590,11 @@ class Document:
                                 self.active_placeholder = None
 
                         self.placeholders.remove(placeholder)
+                        self.ordered_placeholders.remove(placeholder)
+
                         placeholder.remove(force)
 
-                buf.delete_mark(snippet[0])
-                buf.delete_mark(snippet[1])
-                buf.delete_mark(snippet[2])
-                
+                snippet.deactivate()
                 self.active_snippets.remove(snippet)
                 
                 if len(self.active_snippets) == 0:
@@ -592,11 +715,11 @@ class Document:
 
                 # Check for all snippets if the cursor is outside its scope
                 for snippet in list(self.active_snippets):
-                        if snippet[0].get_deleted() or snippet[1].get_deleted():
+                        if snippet.begin_mark.get_deleted() or snippet.end_mark.get_deleted():
                                 self.deactivate(snippet)
                         else:
-                                begin = buf.get_iter_at_mark(snippet[0])
-                                end = buf.get_iter_at_mark(snippet[1])
+                                begin = snippet.begin_iter()
+                                end = snippet.end_iter()
                         
                                 if piter.compare(begin) < 0 or piter.compare(end) > 0:
                                         # Oh no! Remove the snippet this instant!!
@@ -608,13 +731,11 @@ class Document:
                         if self.active_placeholder:
                                 self.jump_placeholders.append((self.active_placeholder, current))
                                 
-                                if self.timeout_update_id != 0:
-                                        gobject.source_remove(self.timeout_update_id)
-                                
-                                self.timeout_update_id = gobject.timeout_add(0, 
-                                                self.update_snippet_contents)
+                                if self.timeout_update_id == 0:
+                                        self.timeout_update_id = gobject.timeout_add(0, 
+                                                        self.update_snippet_contents)
 
-                        self.active_placeholder = current
+                        self.set_active_placeholder(current)
         
         def on_buffer_changed(self, buf):
                 current = self.current_placeholder()
@@ -623,11 +744,48 @@ class Document:
                         if not current in self.update_placeholders:
                                 self.update_placeholders.append(current)
                 
-                        if self.timeout_update_id != 0:
-                                gobject.source_remove(self.timeout_update_id)
-
-                        self.timeout_update_id = gobject.timeout_add(0, \
-                                        self.update_snippet_contents)
+                        if self.timeout_update_id == 0:
+                                self.timeout_update_id = gobject.timeout_add(0, \
+                                                self.update_snippet_contents)
+        
+        def on_buffer_insert_text(self, buf, piter, text, length):
+                ctx = get_buffer_context(buf)
+                
+                # do nothing special if there is no context and no active 
+                # placeholder
+                if (not ctx) and (not self.active_placeholder):
+                        return
+                
+                if not ctx:
+                        ctx = self.active_placeholder
+                
+                if not ctx in self.ordered_placeholders:
+                        return
+                        
+                # move any marks that were incorrectly moved by this insertion
+                # back to where they belong
+                begin = ctx.begin_iter()
+                end = ctx.end_iter()
+                idx = self.ordered_placeholders.index(ctx)
+                
+                for placeholder in self.ordered_placeholders:
+                        if placeholder == ctx:
+                                continue
+                        
+                        ob = placeholder.begin_iter()
+                        oe = placeholder.end_iter()
+                        
+                        if ob.compare(begin) == 0 and ((not oe) or oe.compare(end) == 0):
+                                oidx = self.ordered_placeholders.index(placeholder)
+                                
+                                if oidx > idx and ob:
+                                        buf.move_mark(placeholder.begin, end)
+                                elif oidx < idx and oe:
+                                        buf.move_mark(placeholder.end, begin)
+                        elif ob.compare(begin) >= 0 and ob.compare(end) < 0 and (oe and oe.compare(end) >= 0):
+                                buf.move_mark(placeholder.begin, end)
+                        elif (oe and oe.compare(begin) > 0) and ob.compare(begin) <= 0:
+                                buf.move_mark(placeholder.end, begin)
         
         def on_notify_language(self, buf, spec):
                 self.update_language()
@@ -715,7 +873,8 @@ class Document:
         def apply_uri_snippet(self, snippet, mime, uri):
                 # Remove file scheme
                 if gedit.utils.uri_has_file_scheme(uri):
-                        uri = gnomevfs.get_local_path_from_uri(uri)
+                        gfile = gio.File(uri)
+                        uri = gfile.get_path()
                 
                 # Set environmental variables
                 filename = self.env_get_filename(self.view.get_buffer())
@@ -749,7 +908,7 @@ class Document:
                 
                 for uri in uris:
                         try:
-                                mime = gnomevfs.get_mime_type(uri)
+                                mime = gio.content_type_guess(uri)
                         except:
                                 mime = None
 
